import flet as ft
import asyncio
from datetime import timedelta
from fluxo.settings import AppThemeColors
from fluxo.fluxo_core.database.fluxo import Fluxo as ModelFluxo
from fluxo.fluxo_core.database.task import Task as ModelTask
from fluxo.uttils import convert_str_to_datetime
from fluxo.fluxo_server.screens.home.status_execution import StatusExecution


class Fluxo(ft.UserControl):
    def __init__(self, fluxo: ModelFluxo):
        super().__init__()
        self.fluxo = fluxo

    def build(self):
        self.row_fluxo = ft.Ref[ft.Row]()
        self.text_name = ft.Ref[ft.Text]()
        self.text_interval = ft.Ref[ft.Text]()

        self.container_status = ft.Ref[ft.Container]()
        self.row_executions = ft.Ref[ft.Row]()
        self.text_status = ft.Ref[ft.Text]()

        return ft.Container(
            content=ft.Row(
                ref=self.row_fluxo,
                controls=[
                    ft.Container(
                        content=ft.Stack(
                            controls=[
                                ft.Container(
                                    content=ft.Text(
                                        ref=self.text_name,
                                        weight=ft.FontWeight.BOLD,
                                        color=AppThemeColors.BLACK,
                                        size=13,
                                        width=200
                                    ), # Text
                                    left=0,
                                    top=12
                                ), # Container
                                ft.Container(
                                    content=ft.Text(
                                        ref=self.text_interval,
                                        weight=ft.FontWeight.BOLD,
                                        color=AppThemeColors.BLACK_SECONDARY,
                                        size=10,
                                        width=200
                                    ), # Text
                                    left=150,
                                    top=0,
                                ), # Container
                            ], # controls
                        ), # Stack
                        width=290,
                        height=45,
                    ), # Container
                    ft.Container(
                        content=ft.Row(
                            ref=self.row_executions,
                            controls=[

                            ], # controls
                            width=220,
                            height=50,
                            scroll=ft.ScrollMode.AUTO,
                            auto_scroll=True
                        ), # Row
                    ), # Container
                    ft.Container(width=50),
                    ft.Container(
                        ref=self.container_status,
                        content=ft.Text(
                            ref=self.text_status,
                            weight=ft.FontWeight.BOLD,
                            color=AppThemeColors.WHITE,
                            size=10,
                        ), # Text
                        border_radius=ft.border_radius.all(5),
                        padding=ft.padding.all(5)
                    ), # Container
                ], # controls
                #scroll=ft.ScrollMode.AUTO
            ), # Row
            bgcolor=AppThemeColors.GREY,
            border_radius=ft.border_radius.all(10),
            width=700,
            height=60,
            padding=ft.padding.only(left=15, top=0, right=15, bottom=0)
        ) # Container
    
    async def _load_attributes_fluxo(self):
        # Name
        self.text_name.current.value = self.fluxo.name

        # Interval
        interval = self.fluxo.interval
        if interval.get('minutes'):
            self.text_interval.current.value = f'every {interval.get("minutes")} min(s) at {interval.get("at")[1:]}s'
        elif interval.get('hours'):
            self.text_interval.current.value = f'every {interval.get("hours")} hour(s) at {interval.get("at")[1:]}m'
        elif interval.get('days'):
            self.text_interval.current.value = f'every {interval.get("days")} day(s) at {interval.get("at")}'

        # Status
        if self.fluxo.active:
            self.text_status.current.value = 'Live'
            self.container_status.current.bgcolor = AppThemeColors.GREEN
        else:
            self.text_status.current.value = 'Off'
            self.container_status.current.bgcolor = AppThemeColors.RED

        await self.update_async()

    async def _load_status_executions(self):
        tasks = ModelTask.get_all_by_fluxo_id(self.fluxo.id)

        if tasks:
            # Organize task list by date
            sorted_tasks = sorted(tasks, key=lambda x: x.start_time, reverse=False)

            for task in sorted_tasks:
                self.row_executions.current.controls.append(StatusExecution(self.fluxo, task))
                        
            if len(tasks) < 7:
                n = 7 - len(tasks)
                for _ in range(n):
                    self.row_executions.current.controls.insert(
                        0,
                        ft.Container(
                            bgcolor=AppThemeColors.WHITE,
                            height=23,
                            width=23,
                            border_radius=ft.border_radius.all(15),
                            tooltip=''
                        ), # Container
                    )

        else:
            for _ in range(7):
                self.row_executions.current.controls.insert(
                    0,
                    ft.Container(
                        bgcolor=AppThemeColors.WHITE,
                        height=23,
                        width=23,
                        border_radius=ft.border_radius.all(15),
                        tooltip=''
                    ), # Container
                )


        await self.update_async()

    async def did_mount_async(self):
        self.task_load_attributes_fluxo = asyncio.create_task(self._load_attributes_fluxo())
        self.task_load_status_executions = asyncio.create_task(self._load_status_executions())

    async def will_unmount_async(self):
        self.task_load_attributes_fluxo.cancel()
        self.task_load_status_executions.cancel()


import asyncio
import schedule


class TaskScheduler:
    def __init__(self, coroutines: list):
        '''
        Initializes the TaskScheduler with a list of tuples containing asynchronous tasks and their associated 'Fluxo' information.

        Parameters:
        - coroutines (list): A list of tuples containing asynchronous tasks and their associated 'Fluxo' information.
        '''
        self.coroutines = coroutines

    def schedule_task(self, task):
        '''
        Runs an asynchronous task.

        Parameters:
        - task: The asynchronous task to be executed.
        '''
        asyncio.run(task())

    def run_scheduler(self):
        '''
        Schedules the execution of asynchronous tasks based on the specified intervals.
        '''
        for task, fluxo_info in self.coroutines:
            if fluxo_info.interval.get('minutes'):
                schedule.every(fluxo_info.interval.get('minutes')).minutes.at(
                    fluxo_info.interval.get('at')).do(self.schedule_task, task)
            elif fluxo_info.interval.get('hours'):
                schedule.every(fluxo_info.interval.get('hours')).hours.at(
                    fluxo_info.interval.get('at')).do(self.schedule_task, task)
            elif fluxo_info.interval.get('days'):
                schedule.every(fluxo_info.interval.get('days')).days.at(
                    fluxo_info.interval.get('at')).do(self.schedule_task, task)

        while True:
            schedule.run_pending()



import os
import sqlite3
from fluxo.settings import Db


def _verify_if_db_exists(path_db: str = Db.PATH):
    '''
    Verifies if the database file exists at the specified path. If the file does not exist,
    it creates a new database using the `create_db` function.

    Parameters:
    - path_db (str): The path to the database file. Defaults to the path specified in the
      `settings_db.PATH` variable.

    Returns:
    None
    '''
    if not os.path.exists(path_db):
        create_db(path_db)


def create_db(path_db: str):
    '''
    Creates a SQLite database at the specified path with two tables: 'TB_Fluxo' and 'TB_Task'.
    If the tables do not exist, it creates them along with their respective columns.

    Parameters:
    - path_db (str): The path to the SQLite database file.

    Return:
    None
    '''
    # Database connection
    conn = sqlite3.connect(path_db)

    # Create TB_Fluxo table
    try:
        conn.execute('''
            CREATE TABLE IF NOT EXISTS TB_Fluxo (
                id INTEGER PRIMARY KEY,
                name TEXT,
                date_of_creation DATE,
                interval TEXT,
                active BOOLEAN
            )
        ''')
        print('++ TB_Fluxo table created successfully.')
    except Exception as err:
        raise Exception(f'++ Error creating TB_Fluxo table: {err}')

    # Create TB_Task table
    try:
        conn.execute('''
            CREATE TABLE IF NOT EXISTS TB_Task (
                id INTEGER PRIMARY KEY,
                name TEXT,
                execution_date DATE,
                fluxo_id INTEGER,
                start_time DATE,
                end_time DATE,
                error TEXT,
                FOREIGN KEY (fluxo_id) REFERENCES TB_Fluxo(id) ON DELETE CASCADE
            )
        ''')
        print('++ TB_Task table created successfully.')
    except Exception as err:
        raise Exception(f'++ Error creating TB_Task table: {err}')
    
    # Create TB_App table
    try:
        conn.execute('''
            CREATE TABLE IF NOT EXISTS TB_App (
                id INTEGER PRIMARY KEY,
                active BOOLEAN,
                active_since DATE
            )
        ''')
        print('++ TB_App table created successfully.')
    except Exception as err:
        raise Exception(f'++ Error creating TB_App table: {err}')
    
    finally:
        # Committing the changes
        conn.commit()
        # Closing the database connection
        conn.close()



import flet as ft
from fluxo.settings import AppThemeColors


class AppBar(ft.UserControl):
    def __init__(sefl):
        super().__init__()

    def build(self):
        return ft.Container(
            content=ft.Row(
                controls=[
                    ft.Image(
                        src='icons/icon-512.png',
                        repeat=ft.ImageRepeat.NO_REPEAT,
                        fit=ft.ImageFit.CONTAIN,
                        width=40,
                        height=40,
                    ),
                    ft.Text(
                        value='Fluxo',
                        weight=ft.FontWeight.BOLD,
                        color=AppThemeColors.BLACK,
                        size=25
                    ), # Text
                ]
            ),
            padding=ft.padding.only(left=10, top=5, right=10, bottom=0)
        )


import sqlite3
import json
from datetime import datetime
from dataclasses import dataclass
from fluxo.settings import Db
from fluxo.uttils import current_time_formatted


@dataclass
class Fluxo:
    '''
    Represents a 'Fluxo' object with attributes corresponding to the columns
    in the 'TB_Fluxo' table in the SQLite database.

    Attributes:
    - id (int): The unique identifier for the 'Fluxo'.
    - name (str): The name of the 'Fluxo'.
    - date_of_creation (datetime): The date and time when the 'Fluxo' was created.
    - interval (dict): The interval information for the 'Fluxo'. Ex `{'minutes':1, 'at':':10'}`
    - active (bool): A flag indicating whether the 'Fluxo' is active or not.

    Methods:
    - save(): Saves the current 'Fluxo' instance to the 'TB_Fluxo' table in the database.
    - update(id, name, date_of_creation, interval, active): Updates the 'Fluxo' with the specified ID
      with the provided information in the 'TB_Fluxo' table.
    - get_all(): Retrieves all 'Fluxo' instances from the 'TB_Fluxo' table.
    - get_by_name(name): Retrieves a 'Fluxo' instance by its name from the 'TB_Fluxo' table.
    - get_by_id(id): Retrieves a 'Fluxo' instance by its ID from the 'TB_Fluxo' table.
    - delete(id): Deletes the 'Fluxo' with the specified ID from the 'TB_Fluxo' table.
    - __repr__(): Returns a string representation of the 'Fluxo' instance.
    '''
    id: int = None
    name: str = None
    date_of_creation: datetime = None
    interval: dict = None
    active: bool = True

    def save(self):
        '''
        Saves the current 'Fluxo' instance to the 'TB_Fluxo' table in the database.
        '''
        date_of_creation = current_time_formatted()
        interval = json.dumps(self.interval)

        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('''
            INSERT INTO TB_Fluxo (name, date_of_creation, interval, active)
            VALUES (?, ?, ?, ?)
        ''', (self.name, date_of_creation, interval, self.active))
        conn.commit()
        conn.close()

    @staticmethod
    def update(id, name, date_of_creation, interval, active):
        '''
        Updates the 'Fluxo' with the specified ID with the provided information
        in the 'TB_Fluxo' table.

        Parameters:
        - id (int): The ID of the 'Fluxo' to be updated.
        - name (str): The new name for the 'Fluxo'.
        - date_of_creation (datetime): The new date and time of creation for the 'Fluxo'.
        - interval (dict): The new interval information for the 'Fluxo'.
        - active (bool): The new active status for the 'Fluxo'.
        '''
        interval = json.dumps(interval)

        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('''
            UPDATE TB_Fluxo
            SET name=?, date_of_creation=?, interval=?, active=?
            WHERE id=?
        ''', (name, date_of_creation, interval, active, id))
        conn.commit()
        conn.close()

    @staticmethod
    def get_all():
        '''
        Retrieves all 'Fluxo' instances from the 'TB_Fluxo' table.

        Returns:
        List[Fluxo]: A list containing all 'Fluxo' instances in the database.
        '''
        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('SELECT * FROM TB_Fluxo')
        data = cursor.fetchall()
        conn.close()
        return [Fluxo(*row) for row in data]

    @staticmethod
    def get_by_name(name):
        '''
        Retrieves a 'Fluxo' instance by its name from the 'TB_Fluxo' table.

        Parameters:
        - name (str): The name of the 'Fluxo' to be retrieved.

        Returns:
        Fluxo or None: The 'Fluxo' instance if found, or None if not found.
        '''
        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('SELECT * FROM TB_Fluxo WHERE name=?', (name,))
        data = cursor.fetchone()
        conn.close()
        if data:
            data_list = []
            for value in data:
                data_list.append(value)
            data_list[3] = json.loads(data_list[3])
            return Fluxo(*data_list)
        else:
            return None
        
    @staticmethod
    def get_by_id(id):
        '''
        Retrieves a 'Fluxo' instance by its ID from the 'TB_Fluxo' table.

        Parameters:
        - id (int): The ID of the 'Fluxo' to be retrieved.

        Returns:
        Task or None: The 'Fluxo' instance if found, or None if not found.
        '''
        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('SELECT * FROM TB_Fluxo WHERE id=?', (id,))
        data = cursor.fetchone()
        conn.close()
        if data:
            data_list = []
            for value in data:
                data_list.append(value)
            data_list[3] = json.loads(data_list[3])
            return Fluxo(*data_list)
        else:
            return None

    @staticmethod
    def delete(id):
        '''
        Deletes the 'Fluxo' with the specified ID from the 'TB_Fluxo' table.

        Parameters:
        - id (int): The ID of the 'Fluxo' to be deleted.
        '''
        conn = sqlite3.connect(Db.PATH)
        cursor = conn.cursor()
        cursor.execute('DELETE FROM TB_Fluxo WHERE id=?', (id,))
        conn.commit()
        conn.close()

    def __repr__(self) -> str:
        '''
        Returns a string representation of the 'Fluxo' instance.
        '''
        return f'''
            id:                     {self.id},
            name:                   {self.name},
            date_of_creation:       {self.date_of_creation},
            interval:               {self.interval},
            active:                 {self.active},
        '''



import flet as ft
import asyncio
from fluxo.settings import AppThemeColors
from fluxo.fluxo_server.screens.home.fluxo import Fluxo
from fluxo.fluxo_server.screens.app_bar import AppBar
from fluxo.fluxo_server.screens.footer import Footer
from fluxo.fluxo_core.database.fluxo import Fluxo as ModelFluxo
from fluxo.fluxo_core.database.app import App as ModelApp


class Home(ft.UserControl):
    def __init__(self):
        super().__init__()

    def build(self):
        self.column_fluxos = ft.Ref[ft.Column]()
        self.responsiverow_fluxos = ft.Ref[ft.ResponsiveRow]()
        self.text_status_schedule = ft.Ref[ft.Column]()

        return ft.Column(
            ref=self.column_fluxos,
            controls=[
                ft.Container(height=0),
                ft.Container(
                    content=ft.Row(
                        controls=[
                            ft.Text(
                                value='Fluxos',
                                weight=ft.FontWeight.BOLD,
                                color=AppThemeColors.BLACK,
                                size=25,
                                expand=True
                            ), # Text
                            ft.Tooltip(
                                message='Synchronize',
                                content=ft.IconButton(
                                    icon=ft.icons.SYNC,
                                    icon_color=AppThemeColors.PRIMARY,
                                    on_click=self.on_click_iconbutton_sync
                                ),
                                bgcolor=AppThemeColors.QUARTENARY
                            ),
                            ft.Container(
                                content=ft.Text(
                                    ref=self.text_status_schedule,
                                    text_align=ft.TextAlign.END,
                                    size=12
                                ), # Text
                                bgcolor=AppThemeColors.GREY,
                                border_radius=ft.border_radius.all(10),
                                padding=ft.padding.only(left=5, top=3, right=5, bottom=3)
                            ),
                        ]
                    ),
                    width=700
                ), # Container
                ft.Container(
                    content=ft.ResponsiveRow(
                        ref=self.responsiverow_fluxos,
                        controls=[

                        ], # controls
                        alignment=ft.MainAxisAlignment.CENTER
                    ), # ResponsiveRow
                    alignment=ft.alignment.center,
                    width=700
                ), # Container
            ], # controls
            scroll=ft.ScrollMode.AUTO,
            horizontal_alignment=ft.CrossAxisAlignment.CENTER
        ) # Column
    
    async def _load_fluxos(self):
        self.responsiverow_fluxos.current.controls.clear()
        fluxos = ModelFluxo.get_all()
        if fluxos:
            for fluxo in fluxos:
                fluxo = ModelFluxo.get_by_name(fluxo.name)
                #self.column_fluxos.current.controls.append(Fluxo(fluxo=fluxo))
                self.responsiverow_fluxos.current.controls.append(Fluxo(fluxo=fluxo))
                await self.update_async()

    async def _status_schedule(self):
        app = ModelApp.get()
        if app:
            if app.active:
                self.text_status_schedule.current.value = 'Schedule ON'
                self.text_status_schedule.current.color = AppThemeColors.GREEN
            else:
                self.text_status_schedule.current.value = 'Schedule OFF'
                self.text_status_schedule.current.color = AppThemeColors.RED
        await self.update_async()

    async def on_click_iconbutton_sync(self, e):
        await self.clean_async()
        await self.did_mount_async()


    async def did_mount_async(self):
        self.task_load_fluxos = asyncio.create_task(self._load_fluxos())
        self.task_status_schedule = asyncio.create_task(self._status_schedule())

    async def will_unmount_async(self):
        self.task_load_fluxos.cancel()
        self.task_status_schedule.cancel()


def view_home():
    home = Home()

    return ft.View(
        route='/',
        controls=[
            AppBar(),
            ft.Container(
                content=home,
                expand=True,
                padding=ft.padding.all(15)
            ),
            Footer()
        ],
        padding=ft.padding.all(0),
        bgcolor=AppThemeColors.WHITE,
    )


import flet as ft
import asyncio
from datetime import timedelta
from fluxo.settings import AppThemeColors
from fluxo.fluxo_core.database.fluxo import Fluxo as ModelFluxo
from fluxo.fluxo_core.database.task import Task as ModelTask
from fluxo.uttils import convert_str_to_datetime


class StatusExecution(ft.UserControl):
    def __init__(self, fluxo: ModelFluxo, task: ModelTask):
        super().__init__()
        self.fluxo = fluxo
        self.task = task

    def build(self):
        self.container_execution = ft.Ref[ft.Container]()
        self.tooltip_execution = ft.Ref[ft.Tooltip]()
    
        return ft.Tooltip(
            ref=self.tooltip_execution,
            message='Synchronize',
            content=ft.Container(
                ref=self.container_execution,
                height=23,
                width=23,
                border_radius=ft.border_radius.all(15),
                on_click=self.on_click_task,
            ),
            bgcolor=AppThemeColors.QUARTENARY
        )
    
    async def on_click_task(self, e):
        await self.page.go_async(f'task/{self.task.id}')
    
    async def _load_status_execution(self):
        data_start_time = convert_str_to_datetime(self.task.start_time)
        data_execution_date = convert_str_to_datetime(self.task.execution_date)
        
        if self.task.execution_date is None:
            self.container_execution.current.bgcolor = AppThemeColors.BLUE
            self.tooltip_execution.current.message = self.task.execution_date

        elif self.task.execution_date and self.task.error:
            self.container_execution.current.bgcolor = AppThemeColors.RED
            self.tooltip_execution.current.message = self.task.execution_date

        elif self.fluxo.interval.get('minutes'):
            diference = data_execution_date - data_start_time
            if diference <= timedelta(minutes=self.fluxo.interval.get('minutes')):
                self.container_execution.current.bgcolor = AppThemeColors.GREEN
                self.tooltip_execution.current.message = self.task.execution_date

        elif self.fluxo.interval.get('hours'):
            diference = data_execution_date - data_start_time
            if diference <= timedelta(hours=self.fluxo.interval.get('hours')):
                self.container_execution.current.bgcolor = AppThemeColors.GREEN
                self.tooltip_execution.current.message = self.task.execution_date

        elif self.fluxo.interval.get('days'):
            diference = data_execution_date - data_start_time
            if diference <= timedelta(days=self.fluxo.interval.get('days')):
                self.container_execution.current.bgcolor = AppThemeColors.GREEN
                self.tooltip_execution.current.message = self.task.execution_date

        await self.update_async()

    async def did_mount_async(self):
        self.task_load_status_execution = asyncio.create_task(self._load_status_execution())

    async def will_unmount_async(self):
        self.task_load_status_execution.cancel()




class Fluxo:
    '''
    Represents a 'Fluxo' object with attributes such as name, interval, and active status.

    Attributes:
    - name (str): The name of the 'Fluxo'.
    - interval (dict): The interval information for the 'Fluxo'.
    - active (bool): A flag indicating whether the 'Fluxo' is active or not.
    '''
    def __init__(
        self,
        name: str,
        interval: dict = None,
        active: bool = None
    ):
        self.name = name
        self.interval = interval
        self.active = active



import os
import atexit
import multiprocessing
from fluxo.settings import PathFilesPython
from fluxo.fluxo_core.task import execute_tasks
from fluxo.fluxo_core.database.db import _verify_if_db_exists
from fluxo.fluxo_core.database.app import App as ModelApp


class FluxosExecutor:
    '''
    Represents a 'FluxosExecutor' object responsible for executing Fluxo files in parallel processes.

    Attributes:
    - path (str): The path to the directory containing Fluxo files.
    - processes (list): A list to store the multiprocessing.Process instances.

    Methods:
    - cleanup_processes(): Terminates all running processes when the program exits.
    - execute_fluxos(): Executes Fluxo files in parallel processes.
    '''
    def __init__(self) -> None:
        '''
        Initializes a new 'FluxosExecutor' instance.

        The instance is configured with the path to the directory containing Fluxo files,
        an empty list to store multiprocessing.Process instances, and the verification
        of the existence of the database.

        The cleanup_processes method is registered to be executed on program exit.
        '''
        self.path = PathFilesPython.PATH_FILES_PYTHON
        self.processes = []
        _verify_if_db_exists()

        # Register the functions to be executed on program exit
        atexit.register(self._cleanup_processes)
        atexit.register(self._change_app_status_to_false)

    def _cleanup_processes(self):
        '''
        Terminates all running processes.
        '''
        print("Terminating all processes.")
        for process in self.processes:
            process.terminate()

    def _change_app_status_to_true(self):
        '''
        Changes the application status to True in the database.

        If the application already exists, its status is updated to True.
        Otherwise, a new application with active status set to True is created and saved.
        '''
        app = ModelApp.get()
        if app:
            app.update(app.id, True)
        else:
            app = ModelApp(active=True)
            app.save()

    def _change_app_status_to_false(self):
        '''
        Changes the application status to False in the database.

        If the application exists, its status is updated to False.
        '''
        app = ModelApp.get()
        app.update(app.id, False)

    def execute_fluxos(self):
        '''
        Executes Fluxo files in parallel processes.
        '''
        self._change_app_status_to_true() # Change status to True in database

        for file in os.listdir(self.path):
            if file.endswith(".py"):
                process = multiprocessing.Process(
                    target=execute_tasks, args=(self.path, file,))
                self.processes.append(process)
                process.start()

        for process in self.processes:
            process.join()


if __name__ == '__main__':
    fluxos_executor = FluxosExecutor()
    fluxos_executor.execute_fluxos()


import os
import traceback
import asyncio
import importlib
from datetime import datetime
from fluxo.fluxo_core.fluxo import Fluxo
from fluxo.fluxo_core.database.fluxo import Fluxo as ModelFluxo
from fluxo.fluxo_core.database.task import Task as ModelTask
from fluxo.uttils import current_time_formatted
from fluxo.fluxo_core.scheduler import TaskScheduler


class Task:
    '''
    Represents a 'Task' object associated with a 'Fluxo' for executing asynchronous functions.

    Attributes:
    - task_info (dict): A dictionary containing information about the task, including its name, associated 'Fluxo',
      start time, and end time.

    Methods:
    - __init__(name: str, fluxo: Fluxo, start_time: datetime = None, end_time: datetime = None): Initializes a new 'Task' instance.
    - __call__(func): Decorator method that wraps an asynchronous function, records task information in the database,
      and handles execution details.
    '''
    def __init__(
        self,
        name: str,
        fluxo: Fluxo,
        start_time: datetime = None,
        end_time: datetime = None
    ):
        '''
        Initializes a new 'Task' instance.

        Parameters:
        - name (str): The name of the task.
        - fluxo (Fluxo): The associated 'Fluxo' for the task.
        - start_time (datetime): The start time of the task. Defaults to None.
        - end_time (datetime): The end time of the task. Defaults to None.
        '''
        self.task_info = {
            'name': name,
            'fluxo': fluxo,
            'start_time': start_time,
            'end_time': end_time,
        }

    def __call__(self, func):
        '''
        Decorator method that wraps an asynchronous function, records task information in the database,
        and handles execution details.

        Parameters:
        - func: The asynchronous function to be decorated.

        Returns:
        async function: The decorated asynchronous function.
        '''
        async def wrapper(*args, **kwargs):
            # Retrieve the 'Fluxo' information from the database
            fluxo_register_db = ModelFluxo.get_by_name(
                self.task_info.get('fluxo').name)

            # Create a new 'ModelTask' instance and save it to the database
            task = ModelTask(name=self.task_info.get(
                'name'), fluxo_id=fluxo_register_db.id)
            new_task = task.save()

            try:
                # Call the original function
                new_task.start_time = current_time_formatted()
                new_task.update(**new_task.__dict__)

                result = await func(*args, **kwargs)

                new_task.end_time = current_time_formatted()
                new_task.execution_date = current_time_formatted()
                new_task.update(**new_task.__dict__)

                return result
            except Exception as err:
                error = traceback.format_exc()
                new_task.error = f'[Error in task: {new_task.name}]' + \
                    '\n' + error
                new_task.end_time = current_time_formatted()
                new_task.execution_date = current_time_formatted()
                new_task.update(**new_task.__dict__)

        setattr(wrapper, 'task_info', self.task_info)
        return wrapper


def execute_tasks(path, file):
    '''
    Executes asynchronous tasks defined in a module.

    Parameters:
    - path (str): The path to the directory containing the module.
    - file (str): The name of the module file (excluding the '.py' extension).
    '''
    try:
        # Remove the '.py' extension to get the module name
        name_module = file[:-3]

        # Dynamically import the module
        dir_base = os.path.basename(path)
        module = importlib.import_module(f"{dir_base}.{name_module}")

        coroutines = []

        # Search for asynchronous functions in the module
        for name_attribute in dir(module):
            attribute = getattr(module, name_attribute)
            if asyncio.iscoroutinefunction(attribute):
                # Check if the function is decorated with @Task
                if hasattr(attribute, 'task_info'):

                    task_info = attribute.task_info
                    fluxo_info = task_info.get('fluxo')
                    fluxo_register_db = ModelFluxo.get_by_name(
                        fluxo_info.name)

                    if fluxo_register_db is None:  # Check if fluxo exists in the database
                        fluxo = ModelFluxo(
                            name=fluxo_info.name,
                            interval=fluxo_info.interval)
                        fluxo.save()

                    else:
                        if fluxo_register_db.interval != fluxo_info.interval:
                            fluxo_register_db.interval = fluxo_info.interval
                            fluxo_register_db.update(
                                **fluxo_register_db.__dict__)
                            
                    if ModelFluxo.get_by_name(fluxo_info.name).active: # Check if fluxo.active is True
                        coroutines.append((attribute, fluxo_info))

                else:
                    print("The function is not decorated with @Task")

        if coroutines:
            task_scheduler = TaskScheduler(coroutines)
            task_scheduler.run_scheduler()

    except Exception as e:
        print(f"Error executing task in module {module}: {e}")
